﻿using System;
using System.Reflection;

namespace Common.Commands
{
    public enum ResponseCommand
    {
        [CommandValue("500", 3)]
        CommandAcknowledge = 500,
        [CommandValue("501", 0)]
        CommandError = 501,
        [CommandValue("502", 3)]
        SystemError = 502,
        [CommandValue("505", 1)]
        LoginInteraction = 505,
        [CommandValue("510", 2)]
        KeypadLedStatePartition1Only = 510,
        [CommandValue("511", 2)]
        KeypadLedflashStatePartition1Only = 511,
        [CommandValue("550", 10)]
        TimeDateBroadcast = 550,
        [CommandValue("560", 2)]
        RingDetected = 560,
        [CommandValue("561", 4)]
        IndoorTemperatureBroadcast = 561,
        [CommandValue("562", 4)]
        OutdoorTemperatureBroadcast = 562,
        [CommandValue("601", 4)]
        ZoneAlarm = 601,
        [CommandValue("602", 4)]
        ZoneAlarmRestore = 602,
        [CommandValue("603", 4)]
        ZoneTamper = 603,
        [CommandValue("604", 4)]
        ZoneTamperRestore = 604,
        [CommandValue("605", 3)]
        ZoneFault = 605,
        [CommandValue("606", 3)]
        ZoneFaultRestore = 606,
        [CommandValue("609", 3)]
        ZoneOpen = 609,
        [CommandValue("610", 3)]
        ZoneRestored = 610,
        [CommandValue("615", 256)]
        EnvisalinkZoneTimerDump = 615,
        [CommandValue("620", 4)]
        DuressAlarm = 620,
        [CommandValue("621", 0)]
        FKeyAlarm = 621,
        [CommandValue("622", 0)]
        FKeyRestore = 622,
        [CommandValue("623", 0)]
        AKeyAlarm = 623,
        [CommandValue("624", 0)]
        AKeyRestoral = 624,
        [CommandValue("625", 0)]
        PKeyAlarm = 625,
        [CommandValue("626", 0)]
        PKeyRestore = 626,
        [CommandValue("631", 0)]
        TwoWireSmokeAuxAlarm = 631,
        [CommandValue("632", 0)]
        TwoWireSmokeAuxRestore = 632,
        [CommandValue("650", 1)]
        PartitionReady = 650,
        [CommandValue("651", 1)]
        PartitionNotReady = 651,
        [CommandValue("652", 2)]
        PartitionArmed = 652,
        [CommandValue("653", 1)]
        PartitionReadyForceArmingEnabled = 653,
        [CommandValue("654", 1)]
        PartitionInAlarm = 654,
        [CommandValue("655", 1)]
        PartitionDisarmed = 655,
        [CommandValue("656", 1)]
        ExitDelayinProgress = 656,
        [CommandValue("657", 1)]
        EntryDelayinProgress = 657,
        [CommandValue("658", 1)]
        KeypadLockout = 658,
        [CommandValue("659", 1)]
        PartitionFailedtoArm = 659,
        [CommandValue("660", 1)]
        PgmOutputisinProgress = 660,
        [CommandValue("663", 1)]
        ChimeEnabled = 663,
        [CommandValue("664", 1)]
        ChimeDisabled = 664,
        [CommandValue("670", 1)]
        InvalidAccessCode = 670,
        [CommandValue("671", 1)]
        FunctionNotAvailable = 671,
        [CommandValue("672", 1)]
        FailuretoArm = 672,
        [CommandValue("673", 1)]
        PartitionisBusy = 673,
        [CommandValue("674", 1)]
        SystemArminginProgress = 674,
        [CommandValue("700", 5)]
        UserClosing = 700,
        [CommandValue("701", 1)]
        SpecialClosing = 701,
        [CommandValue("702", 1)]
        PartialClosing = 702,
        [CommandValue("750", 5)]
        UserOpening = 750,
        [CommandValue("751", 1)]
        SpecialOpening = 751,
        [CommandValue("800", 0)]
        PanelBatteryTrouble = 800,
        [CommandValue("801", 0)]
        PanelBatteryTroubleRestore = 801,
        [CommandValue("802", 0)]
        PanelACTrouble = 802,
        [CommandValue("803", 0)]
        PanelACRestore = 803,
        [CommandValue("806", 0)]
        SystemBellTrouble = 806,
        [CommandValue("807", 0)]
        SystemBellTroubleRestoral = 807,
        [CommandValue("814", 0)]
        FtcTrouble = 814,
        [CommandValue("816", 0)]
        BufferNearFull = 816,
        [CommandValue("829", 0)]
        GeneralSystemTamper = 829,
        [CommandValue("830", 0)]
        GeneralSystemTamperRestore = 830,
        [CommandValue("840", 1)]
        TroubleLedon = 840,
        [CommandValue("841", 1)]
        TroubleLedoff = 841,
        [CommandValue("842", 0)]
        FireTroubleAlarm = 842,
        [CommandValue("843", 0)]
        FireTroubleAlarmRestore = 843,
        [CommandValue("849", 2)]
        VerboseTroubleStatus = 849,
        [CommandValue("900", 0)]
        CodeRequired = 900,
        [CommandValue("912", 1)]
        CommandOutputPressed = 912,
        [CommandValue("921", 0)]
        MasterCodeRequired = 921,
        [CommandValue("922", 0)]
        InstallersCodeRequired = 922

    }


    public static class ResponseClass
    {
        public static string GetStringValue(Enum value)
        {
            // Get the type
            Type type = value.GetType();

            // Get fieldinfo for this type
            FieldInfo fieldInfo = type.GetField(value.ToString());

            // Get the stringvalue attributes
            var attribs = fieldInfo.GetCustomAttributes(
                typeof(CommandValueAttribute), false) as CommandValueAttribute[];

            // Return the first if there was a match.
            return attribs.Length > 0 ? attribs[0].CommandValue : null;
        }

        public static int GetNumberOfBytes(Enum value)
        {
            // Get the type
            Type type = value.GetType();

            // Get fieldinfo for this type
            FieldInfo fieldInfo = type.GetField(value.ToString());

            // Get the stringvalue attributes
            var attribs = fieldInfo.GetCustomAttributes(
                typeof(CommandValueAttribute), false) as CommandValueAttribute[];

            // Return the first if there was a match.
            var intValueAsString =  attribs.Length >= 1 ? attribs[1].CommandValue : "0";
            return Int32.Parse(intValueAsString);
        }

        public static void TryParse(string substring, out ResponseCommand command)
        {
            var values = Enum.GetValues(typeof(ResponseCommand));
            foreach( ResponseCommand val in values )
            {
               if (substring.Equals(GetStringValue(val)))
               {
                   command = val;
                   return;
               }
            }
            throw new Exception("ERROR, unknown commandvalue: " + substring);
        }
    }
}
