﻿using System;
using System.Diagnostics;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using Common;
using Common.Commands;
using Logic.Helper;

namespace Logic.Communications
{

    public class Communicator
    {
        private readonly Reciever _reciever;
        private TcpClient _clientSocket;
        private readonly string _envisaLinkdTpiServer;
        private const int TpiPort = 4025;
        private Thread _listenThread;
        private Thread _pollThread;
        public static Communicator Instance;

        public static Communicator GetInstance()
        {
            return Instance; // can be null, than it is not initilized.
        }
        public static Communicator GetCommunicator(string envisaLinkdTpiServer)
        {
            return Instance ?? (Instance = new Communicator(envisaLinkdTpiServer));
        }

        private Communicator(string envisaLinkdTpiServer)
        {
            _envisaLinkdTpiServer = envisaLinkdTpiServer;
            _clientSocket = new TcpClient();
            _reciever = new Reciever(_clientSocket);
        }

        public void Connect()
        {
            _clientSocket.Connect(_envisaLinkdTpiServer, TpiPort);
            _listenThread = new Thread(ListenForData);
            _listenThread.Start(); // start listening thread.
        }

        private void ReConnect()
        {
            //wait 1 second.
            Thread.Sleep(1000);
            _clientSocket = new TcpClient(); // new one
            _reciever.ReplaceClient(_clientSocket);
            _clientSocket.Connect(_envisaLinkdTpiServer, TpiPort);
            _listenThread = new Thread(ListenForData);
            _listenThread.Start(); // start listening thread.
        }

        public void EnablePoll()
        {
            if (_pollThread == null || !_pollThread.IsAlive)
            { // only start if it aint started.
                _pollThread = new Thread(Poll);
                _pollThread.Start(); // start listening thread.
            }
        }

        public void ExecuteRequest(TpiRequest tpiRequest)
        {
            var data = Encoding.ASCII.GetBytes(tpiRequest.Data);
            var serverStream = _clientSocket.GetStream();
            serverStream.Write(data, 0, data.Length);
        }

        public void Subscribe(IResponseHandler responseEventHandler)
        {
            _reciever.Subscribe(responseEventHandler);
        }
        public void UnSubscribe(IResponseHandler responseEventHandler)
        {
            _reciever.UnSubscribe(responseEventHandler);
        }


        private void ListenForData()
        {
            try
            {
                _reciever.Run();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                //nothing, check shutdown flag.
            }

            if (!_reciever.Shutdown) // if not in shutdown mode, try to restart
            {
                ReConnect(); // restart
            }

        }
        private void Poll()
        {
            var r = new TpiRequest(RequestCommand.Poll);
            while (!_reciever.Shutdown)
            {
                ExecuteRequest(r);
                try
                {
                    Thread.Sleep(60 * 1000 * 2); // sleep 2 minutes before next poll
                }
                catch
                {
                    //nothing, check shutdown flag.
                }
            }
        }

        public void Quit()
        {
            _reciever.Shutdown = true;
            if (_pollThread != null)
            {
                _pollThread.Interrupt();
            }
            if (_listenThread != null)
            {
                _listenThread.Interrupt();
            }
        }
        /*
         * execute request and return the response of the specified response
         * If data is not null, it will check the response payload if it starts with the string in data.
         * 
         */
        public IResponse ExecuteRequestAndWaitForResponse(RequestCommand request, ResponseCommand[] response, string data)
        {
            var rg = new ResponseGetter(response, data);
            Subscribe(rg);
            ExecuteRequest(new TpiRequest(RequestCommand.StatusReport));
            return rg.GetResponse();

        }

    }
}
