﻿using System;
using System.Text;
using Common;
using Common.Commands;
using Logic.Exceptions;
using Logic.Helper;

namespace Logic.Communications
{
    public class TpiResponse : EventArgs, IResponse
    {
        private const int CommandSize = 3;
        private const int ChecksumSize = 2;

        public ResponseCommand Command { get; set; }
        public String Data { get; set; }
        public DateTime Date { get; set; }

        public TpiResponse(string data)
        {
            Parse(data);
        }
        public TpiResponse(byte[] data)
        {
            var asString = Encoding.ASCII.GetString(data);
            Parse(asString);
        }

        private void Parse(string data)
        {
            
            if (data == null)
            {
                data = "";
            }
            data = data.Replace("\0", "");
            //check if it's prefixed with date:
            if (data.Length > 9)
            {
                int number;
                Int32.TryParse(data.Substring(0, 1), out number);
                if (number > -1)
                {
                    //prefixed with date.
                    var date = data.Substring(0, 8);
                    data = data.Substring(9); // remove date + space
                    Date = DateTime.Parse(date);
                }
                else
                {
                    Date = new DateTime();
                }
            }

            //get ResponseEnum.
            ResponseCommand command;
            ResponseClass.TryParse(data.Substring(0, CommandSize), out command);
            var numBytes = ResponseClass.GetNumberOfBytes(command);
            var payload = ""; // can be 0-length
            if (numBytes > 0)
            {
                payload = data.Substring(CommandSize, numBytes);
            }
            var checksum = data.Substring(CommandSize + numBytes, ChecksumSize);

            if (!ChecksumHelper.VerifyChecksum(command, payload, checksum))
            {
                throw new WrongChecksumException("Wrong checksum in response detected.");
            }

            Command = command;
            Data = payload;
        }
    }
}
