﻿using System;
using Common.Commands;
using Logic.Communications;
using Plugins;

namespace MainProgram.Tpi
{
    public class CommunicationInitator
    {
        private readonly string _password;
        private readonly string _server;


        public CommunicationInitator(string server, string password)
        {
            _password = password;
            _server = server;
            Init();
        }

        public void Init()
        {
            Communicator.GetCommunicator(_server); // init

            Communicator.GetInstance().Subscribe(new LoginHandler(ContinueWithLogin,
                                                                  WrongPassword));


            try
            {
                Communicator.GetInstance().Connect();
            }
            catch (Exception e)
            {
                //_eventLog.WriteEntry("Error connecting to server\r\n" + e.StackTrace);
                Shutdown();
            }
        }



        public void ContinueWithLogin(LoginHandler obj)
        {
            Communicator.GetInstance().ExecuteRequest(new TpiRequest(RequestCommand.NetworkLogin, _password));
            Communicator.GetInstance().ExecuteRequest(new TpiRequest(RequestCommand.TimeStampControl, "1")); // enable timestamp
            Communicator.GetInstance().EnablePoll(); // start poll (keep alive) thread
            Communicator.GetInstance().ExecuteRequest(new TpiRequest(RequestCommand.StatusReport));
        }
        public void WrongPassword(LoginHandler obj)
        {
            Shutdown();
        }

        public void Shutdown()
        {
            Communicator.GetInstance().Quit();
        }


        public void Log(string message)
        {
            //_eventLog.WriteEntry(message);
        }
    }
}

