﻿using System;
using System.Configuration;
using System.Diagnostics;
using Common;
using Common.Commands;
using Plugins.mail;

namespace Plugins
{
    public class PowerHandler : AbstractResponseEventHandler
    {
        private DateTime _lostPowerTime;
        private readonly MailSender _mailSender;

        private const string LogSource = "Alarm";


        public PowerHandler()
        {
            _mailSender = MailSender.GetInstance();
        }

        public override void EventOccured(IResponse response, IReciever r)
        {
            if (response.Command == ResponseCommand.PanelACTrouble)
            {
                _mailSender.SendMail("Alarm - Power", response.Date + "\r\nLost AC Power");
                _lostPowerTime = response.Date;
            }
            else if (response.Command == ResponseCommand.PanelACRestore)
            {
                var duration = "unknown";
                if (response.Date.Ticks > _lostPowerTime.Ticks)
                {
                    var span = new TimeSpan(response.Date.Ticks - _lostPowerTime.Ticks);
                    var minutes = (int)(span.TotalSeconds / 60);
                    var seconds = (int)(span.TotalSeconds - minutes * 60);
                    duration = minutes + ":" + seconds + "(min:sec)";

                }
                _mailSender.SendMail("Alarm - Power", response.Date + "\r\nAC Power Restored\nDuration: " + duration);
            }
        }
        public string GetName()
        {
            return "PowerHandler";
        }
    }
}
