﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.Net.Mail;
using System.Text;

namespace Plugins.mail
{
    public class MailSender
    {
        private readonly string _smtpServer;
        private readonly string _from;
        private readonly string _to;
        private readonly bool _disabled;

        private static MailSender _instance;
        //private const string LogSource = "Alarm";

        public static MailSender GetInstance()
        {
            if (_instance == null)
            {
                try
                {
                    var v = new AppSettingsReader();
                    var smtpserver = v.GetValue("smtpserver", typeof (string)) as string;
                    var to = v.GetValue("to", typeof (string)) as string;
                    var from = v.GetValue("from", typeof (string)) as string;
                    if (smtpserver != null && to != null && from != null)
                    {
                        //EventLog.WriteEntry(LogSource,"Inited MailSender with server: " + smtpserver + ", to: " + to + ", from: " + from);
                        _instance = new MailSender(smtpserver, from, to);
                    }
                    else
                    {
                        //EventLog.WriteEntry(LogSource, "Missing smtpserver or to or from");
                        _instance = new MailSender();
                    }
                }
                catch (Exception e)
                {
                    //error reading mailconfig.
                    _instance = new MailSender();
                    //EventLog.WriteEntry(LogSource, e.Message);
                }
                _instance.SendMail("Service started", "Service started succesfully");

            }
            return _instance;
        }

        private MailSender()
        {
            _disabled = true;
        }
        private MailSender(string smtpServer, string from, string to)
        {
            _smtpServer = smtpServer;
            _from = from;
            _to = to;
        }

        public void SendMail(string subject, string message)
        {
            if (!_disabled)
            {
                var mailMessage = new MailMessage();
                mailMessage.To.Add(_to);
                mailMessage.Subject = subject;
                mailMessage.From = new MailAddress(_from);
                mailMessage.Body = message;
                var smtp = new SmtpClient(_smtpServer);
                smtp.Send(mailMessage);
            } else
            {
                //EventLog.WriteEntry(LogSource, "Mail disabled, logging instead: \r\n" + subject + "\r\n" + message);
            }

        }
    }
}
